// filename: i2c_eep.c
// KCO 2010.5.28
// for AT24Cxx EEPROM
#include "common.h"
#include "i2c_eep.h"

u16 eep_chk_cnt=0;
EEPROM_Info eeprom;
//----------------------------------------------------   

void i2c_eep_sw_reset(void){
I2C_InitTypeDef I2C_InitStruct;
    I2C_SoftwareResetCmd(I2C2,ENABLE);
    delay_us(10);
    I2C_SoftwareResetCmd(I2C2,DISABLE);
    I2C_InitStruct.I2C_Mode = I2C_Mode_I2C;
    I2C_InitStruct.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStruct.I2C_ClockSpeed = I2C_EEP_SPEED;
    I2C_InitStruct.I2C_DutyCycle = I2C_DutyCycle_2;
    I2C_InitStruct.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStruct.I2C_OwnAddress1 = AT24Cxx_ADDR;
    I2C_Cmd(sEE_I2C, ENABLE);
    I2C_Init(sEE_I2C, &I2C_InitStruct);
}

//----------------------------------------------------   
void init_i2c_eep(int dev_type){
I2C_InitTypeDef I2C_InitStruct;
GPIO_InitTypeDef GPIO_InitStructure;

    /* I2C Clock Enable*/
    RCC_APB2PeriphClockCmd(sEE_I2C_SCL_GPIO_CLK | sEE_I2C_SDA_GPIO_CLK, ENABLE);
    RCC_APB1PeriphClockCmd(sEE_I2C_CLK, ENABLE);

    /*SCL & SDA pin init */
    GPIO_InitStructure.GPIO_Pin = sEE_I2C_SCL_PIN | sEE_I2C_SDA_PIN;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
    GPIO_Init(sEE_I2C_SDA_GPIO_PORT, &GPIO_InitStructure); 
    
   /* sEE_I2C configuration */
    I2C_InitStruct.I2C_Mode = I2C_Mode_I2C;
    I2C_InitStruct.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStruct.I2C_ClockSpeed = I2C_EEP_SPEED;
    I2C_InitStruct.I2C_DutyCycle = I2C_DutyCycle_2;
    I2C_InitStruct.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStruct.I2C_OwnAddress1 = AT24Cxx_ADDR;
    
    I2C_Cmd(sEE_I2C, ENABLE);
    I2C_Init(sEE_I2C, &I2C_InitStruct);
    
    eeprom.DeviceType = dev_type;
    eeprom.DeviceAddress = 0;
    eeprom.InterAddress = 0;
    eeprom.DataLength = 5;
}

//----------------------------------------------------
EEPROM_Status i2c_eep_write(EEPROM_Info *eeprom){
int i;
u8 daddr;
    I2C_GenerateSTART(sEE_I2C,ENABLE);  // START
    
    eep_chk_cnt = 0;
    while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_MODE_SELECT))
      if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    
    switch(eeprom->DeviceType){
    case AT24C01:
    case AT24C02:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0e;
        break;
    case AT24C04:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0c;
        if(eeprom->InterAddress & 0x100) daddr |= 0x02;
        break;
    case AT24C08:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x08;
        if(eeprom->InterAddress & 0x100) daddr |= 0x02;
        if(eeprom->InterAddress & 0x200) daddr |= 0x04;
        break;
    case AT24C16:
        daddr = AT24Cxx_ADDR;
        if(eeprom->InterAddress & 0x100) daddr |= 0x02;
        if(eeprom->InterAddress & 0x200) daddr |= 0x04;
        if(eeprom->InterAddress & 0x400) daddr |= 0x08;
        break;
    case AT24C32:
    case AT24C64:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0e;
        break;
    case AT24C128:
    case AT24C256:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x06;
        break;
    case AT24C512:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0e;
        break;
    case AT24C1024:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0c;
        if(eeprom->InterAddress & 0x10000) daddr |= 0x02;
        break;
    }
    I2C_Send7bitAddress(sEE_I2C, daddr, I2C_Direction_Transmitter);
    
    eep_chk_cnt = 0;
    while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
      if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    
    if(eeprom->DeviceType==AT24C01 || eeprom->DeviceType==AT24C02 || eeprom->DeviceType==AT24C04 
       || eeprom->DeviceType==AT24C08 || eeprom->DeviceType==AT24C16){
        /* Send the EEPROM's internal address to write to */    
        I2C_SendData(sEE_I2C, eeprom->InterAddress);
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
            if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    }
    else if(eeprom->DeviceType==AT24C32 || eeprom->DeviceType==AT24C64 || eeprom->DeviceType==AT24C128 
       || eeprom->DeviceType==AT24C256 || eeprom->DeviceType==AT24C512 || eeprom->DeviceType==AT24C512 
       || eeprom->DeviceType==AT24C1024){
        /* Send the EEPROM's internal address to write to */    
        I2C_SendData(sEE_I2C, eeprom->InterAddress>>8);
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;

        /* Send the EEPROM's internal address to write to */    
        I2C_SendData(sEE_I2C, eeprom->InterAddress);
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    }        

    // write eeprom data    
    for(i=0; i<eeprom->DataLength; i++){
        /* Send the byte to be written */
        I2C_SendData(sEE_I2C, eeprom->Data[i]);
        /* Test on EV8 and clear it */
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    }
    /* Send STOP condition */
    I2C_GenerateSTOP(sEE_I2C, ENABLE); 
    delay_ms(5);
    return EEPROM_SUCCESS; 
}

//----------------------------------------------------
EEPROM_Status i2c_eep_read(EEPROM_Info *eeprom){
int i;
u8 daddr;
    /* Enable Acknowledgement */
    I2C_AcknowledgeConfig(sEE_I2C, ENABLE);

    I2C_GenerateSTART(sEE_I2C,ENABLE);  // start
    eep_chk_cnt = 0;
    while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_MODE_SELECT))
      if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;

    switch(eeprom->DeviceType){
    case AT24C01:
    case AT24C02:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0e;
        break;
    case AT24C04:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0c;
        if(eeprom->InterAddress & 0x100) daddr |= 0x02;
        break;
    case AT24C08:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x08;
        if(eeprom->InterAddress & 0x100) daddr |= 0x02;
        if(eeprom->InterAddress & 0x200) daddr |= 0x04;
        break;
    case AT24C16:
        daddr = AT24Cxx_ADDR;
        if(eeprom->InterAddress & 0x100) daddr |= 0x02;
        if(eeprom->InterAddress & 0x200) daddr |= 0x04;
        if(eeprom->InterAddress & 0x400) daddr |= 0x08;
        break;
    case AT24C32:
    case AT24C64:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0e;
        break;
    case AT24C128:
    case AT24C256:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x06;
        break;
    case AT24C512:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0e;
        break;
    case AT24C1024:
        daddr = AT24Cxx_ADDR | ((eeprom->DeviceAddress)<<1) & 0x0c;
        if(eeprom->InterAddress & 0x10000) daddr |= 0x02;
        break;
    }
    
    I2C_Send7bitAddress(sEE_I2C, daddr, I2C_Direction_Transmitter);  // device addr
    eep_chk_cnt = 0;
    while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED))
      if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
   
    if(eeprom->DeviceType==AT24C01 || eeprom->DeviceType==AT24C02 || eeprom->DeviceType==AT24C04 
       || eeprom->DeviceType==AT24C08 || eeprom->DeviceType==AT24C16){    
        /* Send the internal address to write to */
        I2C_SendData(sEE_I2C, eeprom->InterAddress);
        /* Test on EV8 and clear it */
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    }
    else if(eeprom->DeviceType==AT24C32 || eeprom->DeviceType==AT24C64 || eeprom->DeviceType==AT24C128 
       || eeprom->DeviceType==AT24C256 || eeprom->DeviceType==AT24C512 || eeprom->DeviceType==AT24C1024){    
        /* Send the internal address to write to */    
        I2C_SendData(sEE_I2C, eeprom->InterAddress>>8);
        /* Test on EV8 and clear it */
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
        
        /* Send the internal address to write to */
        I2C_SendData(sEE_I2C, eeprom->InterAddress);
        /* Test on EV8 and clear it */
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_TRANSMITTED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
    }
    
    I2C_GenerateSTART(sEE_I2C, ENABLE);
    eep_chk_cnt = 0;
    while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_MODE_SELECT))
      if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;

    I2C_Send7bitAddress(sEE_I2C, daddr, I2C_Direction_Receiver); // DEVICE ADDR
    eep_chk_cnt = 0;
    while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED))
      if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;

    // read eeprom data
    for(i=0; i<eeprom->DataLength; i++){        
        if(i == eeprom->DataLength - 1){
            /* Disable Acknowledgement */
            I2C_AcknowledgeConfig(sEE_I2C, DISABLE); 
            /* Send STOP Condition */
            I2C_GenerateSTOP(sEE_I2C, ENABLE);      
        }            
        /* Test on EV7 and clear it */
        eep_chk_cnt = 0;
        while(!I2C_CheckEvent(sEE_I2C, I2C_EVENT_MASTER_BYTE_RECEIVED))
          if(++eep_chk_cnt > EEP_MAX_CNT) return EEPROM_ERROR;
        /* Store received data on I2C */
        eeprom->Data[i] = I2C_ReceiveData(sEE_I2C);            
    }
    return EEPROM_SUCCESS;
}
